@@ -4,8 +4,6 @@ import sys
 from collections import defaultdict
 from typing import Union, List, Any, Callable, Type, Dict, Set
 
-from yattag import Doc
-
 from pytconf.color_utils import (
     print_highlight,
     color_hi,
@@ -15,11 +13,10 @@ from pytconf.color_utils import (
     print_error,
 )
 from pytconf.errors_collector import ErrorsCollector
-from pytconf.param import NO_DEFAULT
 from pytconf.param_collector import the_collector
 from pytconf.pydoc import get_first_line
 from pytconf.registry import the_registry
-from pytconf.utils import get_logger
+from pytconf.utils import get_logger, noun, HtmlGen
 
 DEFAULT_FUNCTION_GROUP_NAME = "default"
 DEFAULT_FUNCTION_GROUP_DESCRIPTION = "default command group"
@@ -53,16 +50,6 @@ class Config(metaclass=MetaConfig):
     """
     Base class for all configs
     """
-    pass
-
-
-class HtmlGen:
-    def __init__(self):
-        document, tag, text, line = Doc().ttl()
-        self.document = document
-        self.tag = tag
-        self.text = text
-        self.line = line
 
 
 def is_help(string: str) -> bool:
@@ -73,23 +60,24 @@ class PytconfConf:
     def __init__(self):
         self.main_function: Union[Callable, None] = None
         self.main_description: str = "No application description"
-        self.function_name_to_configs: Dict[str, List[Type[Config]]] = dict()
-        self.function_name_to_suggest_configs: Dict[str, List[Type[Config]]] = dict()
-        self.function_name_to_callable: Dict[str, Callable] = dict()
-        self.allow_free_args: Dict[str, bool] = dict()
-        self.min_free_args: Dict[str, Union[int, None]] = dict()
-        self.max_free_args: Dict[str, Union[int, None]] = dict()
-        self.description: Dict[str, str] = dict()
+        self.function_name_to_configs: Dict[str, List[Type[Config]]] = {}
+        self.function_name_to_suggest_configs: Dict[str, List[Type[Config]]] = {}
+        self.function_name_to_callable: Dict[str, Callable] = {}
+        self.allow_free_args: Dict[str, bool] = {}
+        self.min_free_args: Dict[str, Union[int, None]] = {}
+        self.max_free_args: Dict[str, Union[int, None]] = {}
+        self.description: Dict[str, str] = {}
 
         self.function_group_names: Dict[str, Set[str]] = defaultdict(set)
         self.function_group_list: List[str] = []
-        self.function_group_descriptions: Dict[str, str] = dict()
-        self.function_group_show_meta: Dict[str, bool] = dict()
-        self.function_group_show: Dict[str, bool] = dict()
+        self.function_group_descriptions: Dict[str, str] = {}
+        self.function_group_show_meta: Dict[str, bool] = {}
+        self.function_group_show: Dict[str, bool] = {}
 
         self.free_args: List[str] = []
         self.app_name: str = "No application name"
         self.version: str = "No version"
+        self.default_function = None
 
     def register_main(
         self,
@@ -172,13 +160,10 @@ class PytconfConf:
     def show_help_for_config(cls, config):
         if config == Config:
             return
-        doc = get_first_line(config.__doc__)
-        if doc is not None:
-            print_title(f"  {doc}")
-        else:
-            print_title("  Undocumented parameter set")
+        doc = get_first_line(config, "Undocumented parameter set")
+        print_title(f"  {doc}")
         for name, param in the_registry.yield_name_data_for_config(config):
-            if param.default is NO_DEFAULT:
+            if param.required:
                 default = color_warn("MANDATORY")
             else:
                 default = color_ok(param.t2s(param.default))
@@ -211,11 +196,16 @@ class PytconfConf:
             config = the_registry.get_config_for_name(flag_raw)
             param = the_registry.get_data_for_name(flag_raw)
             edit = value.startswith("=")
-            if edit:
-                v = param.s2t_generate_from_default(value[1:])
-            else:
-                v = param.s2t(value)
-            setattr(config, flag_raw, v)
+            # noinspection PyBroadException
+            # pylint: disable=broad-except
+            try:
+                if edit:
+                    v = param.s2t_generate_from_default(value[1:])
+                else:
+                    v = param.s2t(value)
+                setattr(config, flag_raw, v)
+            except Exception:
+                errors.add_error(f"could not convert [{flag_raw}]")
         if unknown_flags:
             errors.add_error(f"unknown flags [{','.join(unknown_flags)}]")
 
@@ -223,13 +213,12 @@ class PytconfConf:
         missing_parameters = []
         configs = self.function_name_to_configs[function_selected]
         for config in configs:
-            for attribute in the_registry.yield_names_for_config(config):
-                value = getattr(config, attribute)
-                if value is NO_DEFAULT:
-                    missing_parameters.append(attribute)
+            for name, param in the_registry.yield_name_data_for_config(config):
+                if param.required and name not in flags:
+                    missing_parameters.append(name)
         if missing_parameters:
             errors.add_error(
-                f"missing parameters [{','.join(missing_parameters)}]"
+                f"missing {noun('parameter', len(missing_parameters))} [{','.join(missing_parameters)}]"
             )
 
     @staticmethod
@@ -299,7 +288,7 @@ class PytconfConf:
         if args is None:
             args = sys.argv[1:]
 
-        flags: Dict[str, str] = dict()
+        flags: Dict[str, str] = {}
         errors = ErrorsCollector()
         self.free_args = []
 
@@ -329,7 +318,7 @@ class PytconfConf:
                         errors.set_do_help()
                         errors.unset_show_errors()
                     else:
-                        errors.add_error(f"free args are not allowed [{self.free_args}]")
+                        errors.add_error(f"free args are not allowed [{','.join(self.free_args)}]")
 
         if function_selected is None:
             errors.add_error("no command is selected")
@@ -401,14 +390,13 @@ class PytconfConf:
                         self.function_group_names[function_group_name]
                     ):
                         self.get_html_for_function(function_name, html_gen)
-        return html_gen.document.getvalue()
+        # return html_gen.document.getvalue()
+        return html_gen.document
 
     def get_html_for_function(self, function_name, html_gen):
         with html_gen.tag("ul"):
             f = self.function_name_to_callable[function_name]
-            function_doc = get_first_line(f.__doc__)
-            if function_doc is None:
-                function_doc = "not description for this function"
+            function_doc = get_first_line(f, "no description for this function")
             html_gen.line("li", function_name, title="function name: ")
             html_gen.line("li", function_doc, title="function description: ")
             with html_gen.tag("li"):
@@ -421,14 +409,12 @@ class PytconfConf:
     def get_html_for_config(cls, config, html_gen):
         if config == Config:
             return
-        doc = get_first_line(config.__doc__)
-        if doc is None:
-            doc = "undocumented config"
+        doc = get_first_line(config, "undocumented config")
         html_gen.line("h3", doc, title="config: ")
         with html_gen.tag("table"):
             for name, param in the_registry.yield_name_data_for_config(config):
                 with html_gen.tag("tr"):
-                    if param.default is NO_DEFAULT:
+                    if param.required:
                         default = "MANDATORY"
                     else:
                         default = param.t2s(param.default)
@@ -444,11 +430,10 @@ class PytconfConf:
 
     @classmethod
     def write_config_file_json(cls, filename: str) -> None:
-        values: Dict[str, str] = dict()
+        values: Dict[str, str] = {}
         for config in the_registry.yield_configs():
             for name, param in the_registry.yield_name_data_for_config(config):
-                if param.default is not NO_DEFAULT:
-                    values[name] = param.t2s(param.default)
+                values[name] = param.t2s(param.default)
         with open(filename, "wt") as f:
             json.dump(values, f, indent=4)
 
@@ -485,27 +470,24 @@ def get_free_args() -> List[str]:
     return get_pytconf().free_args
 
 
-# TODO: remove default values for next function when ready to force
-# TODO: change function_group_name -> name
-# TODO: change function_group_description -> description
 def register_function_group(
-    function_group_name: str,
-    function_group_description: str,
-    show_meta: bool = True,
-    show: bool = True,
+    name: str,
+    description: str,
+    show_meta: bool,
+    show: bool,
 ) -> None:
     get_pytconf().register_function_group(
-        name=function_group_name,
-        description=function_group_description,
+        name=name,
+        description=description,
         show_meta=show_meta,
         show=show,
     )
 
 
 def register_main(
-    main_description: str = "NO DESCRIPTION",  # TODO: remove when ready to force
-    app_name: str = "NO APP",  # TODO: remove when ready to force
-    version: str = "NO VERSION",  # TODO: remove when ready to force
+    main_description: str,
+    app_name: str,
+    version: str,
 ) -> Callable[[Any], Any]:
     def identity(main_function):
         get_pytconf().register_main(
@@ -520,7 +502,7 @@ def register_main(
 
 
 def register_endpoint(
-    description: str = "No description",  # TODO: remove default value
+    description: str,
     name: Union[str, None] = None,
     configs: List[Type[Config]] = (),
     suggest_configs: List[Type[Config]] = (),
