@@ -2,6 +2,8 @@ import abc
 from enum import Enum
 from typing import List, Union, Callable, Any, Type
 
+from pyfakeuse import fake_use
+
 from pytconf.enum_subset import EnumSubset
 
 from pytconf.extended_enum import str_to_enum_value, enum_type_to_list_str
@@ -12,14 +14,14 @@ from pytconf.convert import convert_str_to_str, convert_str_to_int, convert_int_
     convert_str_to_bool, convert_bool_to_str
 from pytconf.param_collector import the_collector
 
+NO_HELP = "No help for this configuration option"
+
 
 class Unique:
     pass
 
 
 NO_DEFAULT = Unique()
-NO_DEFAULT_TYPE = type(NO_DEFAULT)
-NO_HELP = "No help for this configuration option"
 
 
 class Param(abc.ABC):
@@ -28,10 +30,14 @@ class Param(abc.ABC):
     """
 
     def __init__(
-        self, help_string=NO_HELP, default=NO_DEFAULT, type_name=None,
+        self,
+        help_string=NO_HELP,
+        default: Any = NO_DEFAULT,
+        type_name: str = None,
     ):
         super().__init__()
         self.help_string = help_string
+        self.required = default is NO_DEFAULT
         self.default = default
         self.type_name = type_name
 
@@ -52,9 +58,8 @@ class Param(abc.ABC):
     def t2s(self, t: object) -> str:
         pass
 
-    # noinspection PyMethodMayBeStatic
     def more_help(self) -> Union[str, None]:
-        return None
+        return fake_use(self)
 
 
 class ParamFunctions(Param):
@@ -65,14 +70,16 @@ class ParamFunctions(Param):
     def __init__(
         self,
         help_string=NO_HELP,
-        default=NO_DEFAULT,
+        default=None,
         type_name=None,
         function_s2t: Callable = None,
         function_s2t_generate_from_default: Callable = None,
         function_t2s: Callable = None,
     ):
         super().__init__(
-            help_string=help_string, default=default, type_name=type_name,
+            help_string=help_string,
+            default=default,
+            type_name=type_name,
         )
         self.function_s2t = function_s2t
         self.function_t2s = function_t2s
@@ -92,13 +99,15 @@ class ParamFunctions(Param):
 class ParamFilename(Param):
     def __init__(
         self,
-        help_string=NO_HELP,
-        default=NO_DEFAULT,
+        help_string: str = NO_HELP,
+        default: str = NO_DEFAULT,
         type_name=None,
         suffixes: List[str] = None,
     ):
         super().__init__(
-            help_string=help_string, default=default, type_name=type_name,
+            help_string=help_string,
+            default=default,
+            type_name=type_name,
         )
         self.suffixes = suffixes
         super().collect()
@@ -121,10 +130,15 @@ class ParamFilename(Param):
 
 class ParamEnum(Param):
     def __init__(
-        self, help_string=NO_HELP, default=NO_DEFAULT, enum_type: Type[Enum] = None,
+        self,
+        help_string=NO_HELP,
+        default: Any = NO_DEFAULT,
+        enum_type: Type[Enum] = None,
     ):
         super().__init__(
-            help_string=help_string, default=default, type_name="enum",
+            help_string=help_string,
+            default=default,
+            type_name="enum",
         )
         self.enum_type = enum_type
         super().collect()
@@ -144,10 +158,15 @@ class ParamEnum(Param):
 
 class ParamEnumSubset(Param):
     def __init__(
-        self, help_string=NO_HELP, default=NO_DEFAULT, enum_type: Type[Enum] = None,
+        self,
+        help_string=NO_HELP,
+        default: Any = NO_DEFAULT,
+        enum_type: Type[Enum] = None,
     ):
         super().__init__(
-            help_string=help_string, default=default, type_name="enum",
+            help_string=help_string,
+            default=default,
+            type_name="enum",
         )
         self.enum_type = enum_type
         super().collect()
@@ -170,10 +189,15 @@ class ParamEnumSubset(Param):
 
 class ParamChoice(Param):
     def __init__(
-        self, help_string=NO_HELP, default=NO_DEFAULT, choice_list: List[str] = None
+        self,
+        help_string=NO_HELP,
+        default: str = NO_DEFAULT,
+        choice_list: List[str] = None
     ):
         super().__init__(
-            help_string=help_string, default=default, type_name="Choice",
+            help_string=help_string,
+            default=default,
+            type_name="Choice",
         )
         self.choice_list = choice_list
         super().collect()
@@ -195,7 +219,8 @@ class ParamCreator:
 
     @staticmethod
     def create_int(
-        help_string: str = NO_HELP, default: Union[int, NO_DEFAULT_TYPE] = NO_DEFAULT
+        help_string: str = NO_HELP,
+        default: int = NO_DEFAULT,
     ) -> int:
         """
         Create an int parameter
@@ -211,12 +236,14 @@ class ParamCreator:
             function_t2s=convert_int_to_str,
             function_s2t_generate_from_default=convert_str_to_int_default,
         )
+        if default is NO_DEFAULT:
+            return 0
         return default
 
     @staticmethod
     def create_list_int(
         help_string: str = NO_HELP,
-        default: Union[List[int], NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: List[int] = NO_DEFAULT,
     ) -> List[int]:
         """
         Create a List[int] parameter
@@ -231,12 +258,14 @@ class ParamCreator:
             function_s2t=convert_str_to_list_int,
             function_t2s=convert_list_int_to_str,
         )
+        if default is NO_DEFAULT:
+            return []
         return default
 
     @staticmethod
     def create_list_str(
         help_string: str = NO_HELP,
-        default: Union[List[str], NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: List[str] = NO_DEFAULT,
     ) -> List[str]:
         """
         Create a List[str] parameter
@@ -251,12 +280,14 @@ class ParamCreator:
             function_s2t=convert_str_to_list_str,
             function_t2s=convert_list_str_to_str,
         )
+        if default is NO_DEFAULT:
+            return []
         return default
 
     @staticmethod
     def create_int_or_none(
         help_string: str = NO_HELP,
-        default: Union[int, None, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: Union[int, None] = NO_DEFAULT,
     ) -> Union[int, None]:
         """
         Create an int parameter
@@ -271,11 +302,14 @@ class ParamCreator:
             function_s2t=convert_str_to_int_or_none,
             function_t2s=convert_int_or_none_to_str,
         )
+        if default is NO_DEFAULT:
+            return None
         return default
 
     @staticmethod
     def create_str(
-        help_string: str = NO_HELP, default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT
+        help_string: str = NO_HELP,
+        default: str = NO_DEFAULT,
     ) -> str:
         """
         Create a string parameter
@@ -295,7 +329,7 @@ class ParamCreator:
     @staticmethod
     def create_str_or_none(
         help_string: str = NO_HELP,
-        default: Union[str, None, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: Union[str, None] = NO_DEFAULT,
     ) -> Union[str, None]:
         """
         Create a string parameter
@@ -310,11 +344,14 @@ class ParamCreator:
             function_s2t=convert_str_to_str_or_none,
             function_t2s=convert_str_or_none_to_str,
         )
+        if default == NO_DEFAULT:
+            return None
         return default
 
     @staticmethod
     def create_bool(
-        help_string: str = NO_HELP, default: Union[bool, NO_DEFAULT_TYPE] = NO_DEFAULT
+        help_string: str = NO_HELP,
+        default: bool = NO_DEFAULT,
     ) -> bool:
         """
         Create a bool parameter
@@ -329,13 +366,15 @@ class ParamCreator:
             function_s2t=convert_str_to_bool,
             function_t2s=convert_bool_to_str,
         )
+        if default is NO_DEFAULT:
+            return False
         return default
 
     @staticmethod
     def create_new_file(
         help_string: str = NO_HELP,
-        default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT,
-        suffixes: Union[List[str], None] = None,
+        default: str = NO_DEFAULT,
+        suffixes: List[str] = None,
     ) -> str:
         """
         Create a new file parameter
@@ -350,12 +389,14 @@ class ParamCreator:
             type_name="new_file",
             suffixes=suffixes,
         )
+        if default is NO_DEFAULT:
+            return ""
         return default
 
     @staticmethod
     def create_existing_file(
         help_string: str = NO_HELP,
-        default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: str = NO_DEFAULT,
         suffixes: Union[List[str], None] = None,
     ) -> str:
         """
@@ -371,12 +412,14 @@ class ParamCreator:
             type_name="existing_file",
             suffixes=suffixes,
         )
+        if default is NO_DEFAULT:
+            return ""
         return default
 
     @staticmethod
     def create_existing_folder(
         help_string: str = NO_HELP,
-        default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: str = NO_DEFAULT,
         suffixes: Union[List[str], None] = None,
     ) -> str:
         """
@@ -392,13 +435,15 @@ class ParamCreator:
             type_name="existing_folder",
             suffixes=suffixes,
         )
+        if default is NO_DEFAULT:
+            return ""
         return default
 
     @staticmethod
     def create_choice(
         choice_list: List[str],
         help_string: str = NO_HELP,
-        default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: str = NO_DEFAULT,
     ) -> str:
         """
         Create a choice config
@@ -408,15 +453,19 @@ class ParamCreator:
         :return:
         """
         ParamChoice(
-            help_string=help_string, default=default, choice_list=choice_list,
+            help_string=help_string,
+            default=default,
+            choice_list=choice_list,
         )
+        if default is NO_DEFAULT:
+            return ""
         return default
 
     @staticmethod
     def create_enum(
         enum_type: Type[Enum],
         help_string: str = NO_HELP,
-        default: Union[Any, NO_DEFAULT_TYPE] = NO_DEFAULT,
+        default: Type[Enum] = NO_DEFAULT,
     ) -> Type[Enum]:
         """
         Create an enum config
@@ -425,14 +474,20 @@ class ParamCreator:
         :param default:
         :return:
         """
-        ParamEnum(help_string=help_string, default=default, enum_type=enum_type,)
+        ParamEnum(
+            help_string=help_string,
+            default=default,
+            enum_type=enum_type,
+        )
+        if default is NO_DEFAULT:
+            return enum_type
         return default
 
     @staticmethod
     def create_enum_subset(
         enum_type: Type[Enum],
         help_string: str = NO_HELP,
-        default: Union[Any, EnumSubset] = NO_DEFAULT,
+        default: EnumSubset = NO_DEFAULT,
     ) -> EnumSubset:
         """
         Create an enum config
@@ -442,13 +497,18 @@ class ParamCreator:
         :return:
         """
         ParamEnumSubset(
-            help_string=help_string, default=default, enum_type=enum_type,
+            help_string=help_string,
+            default=default,
+            enum_type=enum_type,
         )
+        if default is NO_DEFAULT:
+            return EnumSubset(enum_type=enum_type, list_of_values=[])
         return default
 
     @staticmethod
     def create_existing_bucket(
-        help_string: str = NO_HELP, default: Union[str, NO_DEFAULT_TYPE] = NO_DEFAULT
+        help_string: str = NO_HELP,
+        default: str = NO_DEFAULT,
     ) -> str:
         """
         Create a bucket name on gcp
@@ -463,4 +523,6 @@ class ParamCreator:
             function_s2t=convert_str_to_str,
             function_t2s=convert_str_to_str,
         )
+        if default is NO_DEFAULT:
+            return ""
         return default
